@@ -212,7 +212,7 @@ cal_backend_wcap_tasks_update_objects (ECalBackendSync *backend, EDataCal *cal,
 			return_subcomp = icalcomponent_get_first_component (return_icalcomp,
 									    child_kind);
 			while (return_subcomp) {
-				cal_backend_wcap_add_component (wcap, return_subcomp, TRUE, old_object);
+				cal_backend_wcap_add_component (wcap, return_subcomp, TRUE, old_object, NULL);
 				return_subcomp = icalcomponent_get_next_component (return_icalcomp,
 										   child_kind);
 			}
