@@ -28,11 +28,11 @@
 #include <libical/ical.h>
 #include <libedataserver/e-xml-hash-utils.h>
 #include <libedataserver/e-data-server-util.h>
-#include "lib/sunone-account.h"
 #include "lib/sunone-ace.h"
 #include "lib/sunone-util.h"
 #include "storage/sunone-component.h"
 #include "storage/sunone-config-listener.h"
+#include "storage/sunone-account.h"
 #include "cal-backend-wcap.h"
 
 struct _CalBackendWCAPPrivate {
@@ -45,6 +45,7 @@ struct _CalBackendWCAPPrivate {
 	SunOneCompType type;
 	CalMode mode;
 	char *cache_name;
+	ECalBackendSyncStatus open_error;
 	
 	gint timeout_id;
 	time_t timestamp;
@@ -729,6 +730,7 @@ process_component_from_server (CalBackendWCAP *wcap, ECalComponent *comp)
 		dt_start.tzid = g_strdup (icaltimezone_get_tzid (priv->default_zone));
 		e_cal_component_set_dtstart (comp, &dt_start);
 		if (!dt_end.value) {
+			dt_start.value->day = dt_start.value->day + 1;
 			e_cal_component_set_dtend (comp, &dt_start);
 		} else if (dt_end.value && dt_end.value->is_date) {
 			g_free ((char*)dt_end.tzid);
@@ -807,11 +809,44 @@ process_component_from_server (CalBackendWCAP *wcap, ECalComponent *comp)
 		e_cal_component_set_organizer (comp, NULL);
 	} 
 	
+	/* Alarms */
+	if (e_cal_component_has_alarms (comp)) {
+		ECalComponentDateTime dtstart;
+		icaltimezone *utc_zone = icaltimezone_get_utc_timezone ();
+		
+		e_cal_component_get_dtstart (comp, &dtstart);
+		if (dtstart.value) {
+			GList *auids, *l;
+			dtstart.value->is_date = 0;
+			icaltimezone_convert_time (dtstart.value, cal_backend_wcap_get_default_zone (wcap), utc_zone);
+			
+			auids = e_cal_component_get_alarm_uids (comp);
+			for (l = auids; l != NULL; l = l->next) {
+				const char *auid = l->data;
+				ECalComponentAlarm *alarm;
+				ECalComponentAlarmTrigger trigger;
+				
+				alarm = e_cal_component_get_alarm (comp, auid);
+				e_cal_component_alarm_get_trigger (alarm, &trigger);
+				if (trigger.type == E_CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE) {
+					struct icaldurationtype duration = icaltime_subtract (trigger.u.abs_time, *dtstart.value);
+
+					trigger.u.rel_duration = duration;
+					trigger.type = E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_START;
+					e_cal_component_alarm_set_trigger (alarm, trigger);
+				}
+				e_cal_component_alarm_free (alarm);
+			}
+			cal_obj_uid_list_free (auids);
+		}
+		e_cal_component_free_datetime (&dtstart);
+	}
+
 	e_cal_component_abort_sequence (comp);
 }
 
 void
-cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, gboolean process, char **old_object)
+cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, gboolean process, char **old_object, char **new_object)
 {
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	icalcomponent_kind kind;
@@ -841,8 +876,8 @@ cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, g
 		
 		l = g_list_find_custom (ilist, comp, cal_backend_wcap_instance_list_compare);
 		if (l) {
-			if (old_object)
-				*old_object = icalcomponent_as_ical_string (e_cal_component_get_icalcomponent (l->data));
+			if (old_object && !*old_object)
+				*old_object = e_cal_component_get_as_string (l->data);
 			ilist = g_list_remove_link (ilist, l);
 		}
 		ilist = g_list_insert_sorted (ilist, comp, cal_backend_wcap_instance_list_compare);
@@ -861,13 +896,12 @@ cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, g
 	
 	/* Track the mangled uid components */
 	if (g_hash_table_lookup_extended (priv->objects, uid, &orig_key, &orig_value)) {
-		ECalComponentId id;
-		char *calobj = g_strdup (icalcomponent_as_ical_string (e_cal_component_get_icalcomponent (orig_value)));
+		ECalComponentId *id;
+		char *calobj = e_cal_component_get_as_string (orig_value);
 
-		id.uid = g_strdup (uid);
-		id.rid = NULL;
-		e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), &id, calobj, NULL);
-		g_free (id.uid);
+		id = e_cal_component_get_id (comp);
+		e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), id, calobj, NULL);
+		e_cal_component_free_id (id);
 		g_free (calobj);
 		g_hash_table_remove (priv->objects, uid);
 		g_free (orig_key);
@@ -881,8 +915,10 @@ cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, g
 
 	g_hash_table_insert (priv->objects, g_strdup (uid), comp);
 
-	char *calobj = g_strdup(icalcomponent_as_ical_string (e_cal_component_get_icalcomponent (comp)));
+	char *calobj = e_cal_component_get_as_string (comp);
 	if(calobj) {
+		if (new_object && !*new_object)
+			*new_object = g_strdup (calobj);
 		e_cal_backend_notify_object_created (E_CAL_BACKEND (wcap), calobj);
 	}
 	g_free (calobj);
@@ -901,7 +937,7 @@ add_toplevel_component (CalBackendWCAP *wcap, icalcomponent *toplevel, gboolean
 			kind = icalcomponent_isa (subcomp);
 			if (kind == ICAL_VEVENT_COMPONENT ||
 			    kind == ICAL_VTODO_COMPONENT) {
-				cal_backend_wcap_add_component (wcap, subcomp, process, NULL);
+				cal_backend_wcap_add_component (wcap, subcomp, process, NULL, NULL);
 			} else if (kind == ICAL_VTIMEZONE_COMPONENT) {
 				add_timezone (wcap, subcomp);
 			}
@@ -909,14 +945,14 @@ add_toplevel_component (CalBackendWCAP *wcap, icalcomponent *toplevel, gboolean
 		}
 	} else if (kind == ICAL_VEVENT_COMPONENT ||
 		   kind == ICAL_VTODO_COMPONENT) {
-		cal_backend_wcap_add_component (wcap, toplevel, process, NULL);
+		cal_backend_wcap_add_component (wcap, toplevel, process, NULL, NULL);
 	} else if (kind == ICAL_VTIMEZONE_COMPONENT) {
 		add_timezone (wcap, toplevel);
 	}
 }
 
 void
-cal_backend_wcap_remove_component (CalBackendWCAP *wcap, const char *uid, CalObjModType mod)
+cal_backend_wcap_remove_component (CalBackendWCAP *wcap, const char *uid, CalObjModType mod, char **old_object)
 {
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	ECalComponent *comp;
@@ -963,14 +999,15 @@ cal_backend_wcap_remove_component (CalBackendWCAP *wcap, const char *uid, CalObj
 		
 		e_cal_component_get_uid (remove_comp, &mangled_uid);
 		if (g_hash_table_lookup_extended (priv->objects, mangled_uid, &orig_key, &orig_value)) {
-			ECalComponentId id;
-			char *calobj = g_strdup (icalcomponent_as_ical_string (e_cal_component_get_icalcomponent (remove_comp)));
+			ECalComponentId *id;
+			char *calobj = e_cal_component_get_as_string (remove_comp);
 
-			id.uid = g_strdup (mangled_uid);
-			id.rid = NULL;
-			e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), &id, calobj, NULL);
-			g_free (id.uid);
+			if (old_object && !*old_object)
+				*old_object = g_strdup (calobj);
 			g_hash_table_remove (priv->objects, mangled_uid);
+			id = e_cal_component_get_id (remove_comp);
+			e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), id, calobj, NULL);
+			e_cal_component_free_id (id);
 			g_free (calobj);
 			g_free (orig_key);
 			g_object_unref (G_OBJECT (orig_value));
@@ -1000,7 +1037,7 @@ remove_icalcomp_list (CalBackendWCAP *wcap, icalcomponent *icalcomp)
 			e_cal_component_set_icalcomponent (comp, subcomp);
 			sunone_util_mangle_uid (comp);
 			e_cal_component_get_uid (comp, &mangled_uid);
-			cal_backend_wcap_remove_component (wcap, mangled_uid, CALOBJ_MOD_THIS);
+			cal_backend_wcap_remove_component (wcap, mangled_uid, CALOBJ_MOD_THIS, NULL);
 			e_cal_component_set_icalcomponent (comp, NULL);
 		}
 		subcomp = icalcomponent_get_next_component (icalcomp,
@@ -1388,6 +1425,9 @@ cal_backend_wcap_poll_cb (gpointer data)
 	icalcomponent *icalcomp;
 	guint error_code;
 	
+	if (!priv->so_cnc)
+		return FALSE;
+
 	utc_zone = icaltimezone_get_utc_timezone ();
 
 	/* Get the latest properties */
@@ -1644,7 +1684,7 @@ cal_backend_wcap_get_alarm_email_address (ECalBackendSync *backend, EDataCal *ca
 
 	d(printf("cal_backend_wcap_get_alarm_email_address(%p, %s)\n", backend, *address));
 
-	if (!priv->alarm_email_address) {
+	if (!priv->alarm_email_address && priv->so_cnc) {
 		prefs = sunone_connection_get_preferences (priv->so_cnc);
 		if (prefs->alarm_mail)
 			priv->alarm_email_address = g_strdup (prefs->alarm_mail);
@@ -1701,39 +1741,36 @@ cal_backend_wcap_get_static_capabilities (ECalBackendSync *backend, EDataCal *ca
 }
 
 static ECalBackendSyncStatus 
-go_online (CalBackendWCAP *wcap)
+go_online_with_thread (CalBackendWCAP *wcap, SunOneAccount *account)
 {
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	icalcomponent *icalcomp;
-	SunOneAccount *account;
 	SunOneConnection *cnc;
 	guint error_code;
 	SunOneConnectionPreferences *prefs;
 	char *key;
 	const char *uristr;
+	gboolean readonly;
 
 	g_mutex_lock (priv->open_lock);
 
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
-	account = sunone_component_get_account_from_uri (global_sunone_component, uristr);
-	if (!account) {
-		g_mutex_unlock (priv->open_lock);
-		return GNOME_Evolution_Calendar_OtherError;
-	}
 	cnc = sunone_account_get_connection (account);
 	if (!cnc) {
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
-		return GNOME_Evolution_Calendar_OtherError;
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
+		priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return priv->open_error;
 	}
 
 	if (priv->calid)
 		g_free (priv->calid);
 	priv->calid = sunone_util_get_calid_from_uri (uristr);
 	if (!priv->calid) {
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
-		return GNOME_Evolution_Calendar_OtherError;
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
+		priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return priv->open_error;
 	}
 	if (priv->uri)
 		g_free (priv->uri);
@@ -1742,16 +1779,16 @@ go_online (CalBackendWCAP *wcap)
 		g_free (priv->cache_name);
 	priv->cache_name = get_cache_name (wcap, TRUE);
 	priv->so_cnc = g_object_ref (cnc);
-	read_cache (wcap);
 
 	/* load timezones */
 	error_code = sunone_connection_get_all_timezones (priv->so_cnc, &icalcomp);
 
 	if (!SUNONE_ERROR_IS_SUCCESSFUL (error_code)) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
-		return GNOME_Evolution_Calendar_OtherError;
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
+		priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return priv->open_error;
 	}
 
 	add_toplevel_component (wcap, icalcomp, TRUE, FALSE);
@@ -1764,18 +1801,20 @@ go_online (CalBackendWCAP *wcap)
 	priv->props = sunone_connection_get_calprops (priv->so_cnc, priv->calid, FALSE);
 	if (!priv->props) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
-		return GNOME_Evolution_Calendar_OtherError;
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
+		priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return priv->open_error;
 	}
 	
 	/* Get the appropriate timezone for this calendar */
 	prefs = sunone_connection_get_preferences (priv->so_cnc);
 	if (!prefs) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
-		return GNOME_Evolution_Calendar_OtherError;
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
+		priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return priv->open_error;
 	}
 
 	if (prefs->single_calendar_tzid && priv->props->timezone) {
@@ -1806,15 +1845,19 @@ go_online (CalBackendWCAP *wcap)
 								   &icalcomp, &priv->timestamp);
 	if (!SUNONE_ERROR_IS_SUCCESSFUL (error_code)) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
 
-		if (error_code == SUNONE_ERROR_ACCESS_DENIED)
-			return GNOME_Evolution_Calendar_PermissionDenied;
-		else if (error_code == SUNONE_ERROR_LOGIN_OK_DEFAULT_CALENDAR_NOT_FOUND)
-			return GNOME_Evolution_Calendar_NoSuchCal;
-
-		return GNOME_Evolution_Calendar_OtherError;
+		if (error_code == SUNONE_ERROR_ACCESS_DENIED) {
+			priv->open_error = GNOME_Evolution_Calendar_PermissionDenied;
+			return priv->open_error;
+		} else if (error_code == SUNONE_ERROR_LOGIN_OK_DEFAULT_CALENDAR_NOT_FOUND) {
+			priv->open_error = GNOME_Evolution_Calendar_NoSuchCal;
+			return priv->open_error;
+		} else {
+			priv->open_error = GNOME_Evolution_Calendar_OtherError;
+			return priv->open_error;
+		}
 	}
 
 	add_toplevel_component (wcap, icalcomp, TRUE, FALSE);
@@ -1830,8 +1873,10 @@ go_online (CalBackendWCAP *wcap)
 	sunone_connection_add_wcap (priv->so_cnc, key, (gpointer)wcap);
 	g_free (key);
 
+	cal_backend_wcap_is_read_only (E_CAL_BACKEND_SYNC(wcap), NULL, &readonly);
+	e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), readonly);
+
 	priv->timeout_id = g_timeout_add (sunone_account_get_poll_interval (account) * 60 * 1000, cal_backend_wcap_poll_cb, wcap);
-	g_object_unref (account);
 
 	cal_backend_wcap_write_cache (wcap);
 
@@ -1840,6 +1885,28 @@ go_online (CalBackendWCAP *wcap)
 	return GNOME_Evolution_Calendar_Success;
 }
 
+static ECalBackendSyncStatus 
+go_online (CalBackendWCAP *wcap)
+{
+	SunOneAccount *account;
+	ECalBackendSyncStatus ret;
+	const char *uristr;
+
+	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
+	account = sunone_component_get_account_from_uri (global_sunone_component, uristr);
+	if (!account) {
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
+		wcap->priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return wcap->priv->open_error;
+	}
+	sunone_account_set_connecting (account, TRUE);
+	ret = go_online_with_thread (wcap, account);
+	sunone_account_set_connecting (account, FALSE);
+	g_object_unref (account);
+
+	return ret;
+}
+
 static ECalBackendSyncStatus
 cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 			const char *username, const char *password)
@@ -1849,7 +1916,7 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 	icalcomponent_kind kind;
 	GThread *thread = NULL;
 	GError *error = NULL;
-	int ret;
+	const char *uristr;
 
 	d(printf("cal_backend_wcap_open(%p, %d)\n", backend, only_if_exists));
 
@@ -1861,20 +1928,20 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 	if (kind == ICAL_VTODO_COMPONENT)
 		priv->type = TYPE_TODO;
 	else if (kind == ICAL_VEVENT_COMPONENT)
-		 priv->type = TYPE_EVENT;
+		priv->type = TYPE_EVENT;
+
+	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
+	priv->calid = sunone_util_get_calid_from_uri (uristr);
+	if (!priv->calid)
+		return GNOME_Evolution_Calendar_NoSuchCal;
+
+	priv->uri = g_strdup (uristr);
+	priv->cache_name = get_cache_name (wcap, TRUE);
 
 	if (priv->mode == CAL_MODE_LOCAL) {
 		ESource *source;
-		const char *uristr;
 		const char *display_contents = NULL;
 
-		uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
-		priv->calid = sunone_util_get_calid_from_uri (uristr);
-		if (!priv->calid)
-			return GNOME_Evolution_Calendar_NoSuchCal;
-		priv->uri = g_strdup (uristr);
-		priv->cache_name = get_cache_name (wcap, TRUE);
-
 		source = e_cal_backend_get_source (E_CAL_BACKEND (backend));
 		display_contents = e_source_get_property (source, "offline_sync");
 		if (!display_contents || !g_str_equal (display_contents, "1")) {
@@ -1886,6 +1953,7 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 
 		return GNOME_Evolution_Calendar_Success;
 	}
+	read_cache (wcap);
 
 	thread = g_thread_create ((GThreadFunc)go_online, wcap, FALSE, &error);
 	if (!thread) {
@@ -2213,7 +2281,7 @@ struct cbw_data {
 };
 
 static void
-e_cal_backend_wcap_compute_changes_foreach_key (const char *key, gpointer data)
+e_cal_backend_wcap_compute_changes_foreach_key (const char *key, gpointer value, gpointer data)
 {
 	struct cbw_data *cbwdata = data;
 	ECalComponent *comp = NULL;
@@ -2229,13 +2297,16 @@ e_cal_backend_wcap_compute_changes_foreach_key (const char *key, gpointer data)
 			e_cal_component_set_new_vtype (comp, E_CAL_COMPONENT_TODO);
 		else if (cbwdata->kind == ICAL_VEVENT_COMPONENT)
 			e_cal_component_set_new_vtype (comp, E_CAL_COMPONENT_EVENT);
-		else
+		else {
+			g_object_unref (comp);
 			return;
+		}
 		
 		e_cal_component_set_uid (comp, key);
 		cbwdata->deletes = g_list_prepend (cbwdata->deletes, e_cal_component_get_as_string (comp));
 
 		e_xmlhash_remove (cbwdata->ehash, key);
+		g_object_unref (comp);
 	}
 }
 
@@ -2268,11 +2339,11 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 	if (!account)
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 	
-	if (kind ==  E_CAL_COMPONENT_EVENT)
+	if (kind ==  ICAL_VEVENT_COMPONENT)
 		dirname = g_strdup_printf ("%s/sunone/%s@%s/Calendar", evolution_dir,
 					   sunone_account_get_user (account),
 					   sunone_account_get_server (account));
-	else if (kind == E_CAL_COMPONENT_TODO)
+	else if (kind == ICAL_VTODO_COMPONENT)
 		dirname = g_strdup_printf ("%s/sunone/%s@%s/Tasks", evolution_dir,
 					   sunone_account_get_user (account),
 					   sunone_account_get_server (account));
@@ -2300,7 +2371,7 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 	data.ehash = xmlhash;
 	data.adds = NULL;
 	data.modifies = NULL;
-	if (kind == E_CAL_COMPONENT_EVENT || kind == E_CAL_COMPONENT_TODO)
+	if (kind == ICAL_VEVENT_COMPONENT || kind == ICAL_VTODO_COMPONENT)
 		g_hash_table_foreach (priv->objects, check_change_type, &data);
 	else
 		return GNOME_Evolution_Calendar_OtherError;
@@ -2421,9 +2492,11 @@ cal_backend_wcap_set_mode (ECalBackend *backend, CalMode mode)
 	if (mode == CAL_MODE_REMOTE) {
 		if (priv->uri) {
 			account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
-			sunone_account_set_online (account);
-			status = go_online (wcap);
-			g_object_unref (account);
+			if (account) {
+				sunone_account_set_online (account);
+				status = go_online (wcap);
+				g_object_unref (account);
+			}
 		}
 	} else if (mode == CAL_MODE_LOCAL) {
 		if (priv->so_cnc && sunone_connection_is_open (priv->so_cnc))
@@ -2431,8 +2504,10 @@ cal_backend_wcap_set_mode (ECalBackend *backend, CalMode mode)
 		in_offline (wcap);
 		if (priv->uri) {
 			account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
-			sunone_account_set_offline (account);
-			g_object_unref (account);
+			if (account) {
+				sunone_account_set_offline (account);
+				g_object_unref (account);
+			}
 		}
 	} else {
 		e_cal_backend_notify_mode (backend, GNOME_Evolution_Calendar_CalListener_MODE_NOT_SUPPORTED, cal_mode_to_corba (mode));
@@ -2489,6 +2564,14 @@ cal_backend_wcap_result_from_error (guint error_code)
 	return GNOME_Evolution_Calendar_InvalidObject;
 }
 
+ECalBackendSyncStatus
+cal_backend_wcap_get_open_error (CalBackendWCAP *wcap)
+{
+	g_return_val_if_fail (IS_CAL_BACKEND_WCAP (wcap), GNOME_Evolution_Calendar_OtherError);
+
+	return wcap->priv->open_error;
+}
+
 static ECalBackendSyncStatus
 cal_backend_wcap_get_timezone (ECalBackendSync *backend, EDataCal *cal,
 			            const char *tzid, char **object)
@@ -2569,6 +2652,7 @@ cal_backend_wcap_init (CalBackendWCAP *object)
 
 	priv->type = TYPE_ALL;
 	priv->mode = CAL_MODE_REMOTE;
+	priv->open_error = GNOME_Evolution_Calendar_OtherError;
 
 	priv->timeout_id = -1;
 
