@@ -48,7 +48,7 @@ sunone_component_factory (BonoboGenericFactory *factory,
 	return component;
 }
 
-static void
+static gboolean
 setup_component_factory (void)
 {
 	global_sunone_component = sunone_component_new ();
@@ -56,6 +56,8 @@ setup_component_factory (void)
 	component_factory = bonobo_generic_factory_new (SUNONE_COMPONENT_FACTORY_IID,
 				sunone_component_factory,
 				global_sunone_component);
+
+	return TRUE;
 }
 
 static void
@@ -106,7 +108,7 @@ remove_group (ESourceList *source_list)
 	for (p = groups; p != NULL; p = g_slist_next (p)) {
 		ESourceGroup *group = E_SOURCE_GROUP (p->data);
 		base_uri = e_source_group_peek_base_uri (group);
-		if (strcmp (base_uri, "wcap://") == 0) {
+		if (strncmp (base_uri, "wcap://", 7) == 0) {
 			to_delete = g_slist_append (to_delete, group);
 		}
 	}
@@ -114,8 +116,8 @@ remove_group (ESourceList *source_list)
 	for (p = to_delete; p!= NULL; p = g_slist_next (p)) {
 		ESourceGroup *group = E_SOURCE_GROUP (p->data);
 		e_source_list_remove_group (source_list, group);
-		e_source_list_sync (source_list, NULL);
 	}
+	e_source_list_sync (source_list, NULL);
 
 	g_slist_free (to_delete);
 }
@@ -182,7 +184,8 @@ main (int argc, char *argv[])
 	master_client = gnome_master_client ();
 	gnome_client_set_restart_style (master_client, GNOME_RESTART_NEVER);
 
-	setup_component_factory ();
+	if (!setup_component_factory ())
+		goto failed;
 	if (!setup_calendar ())
 		goto failed;
 
@@ -192,6 +195,17 @@ main (int argc, char *argv[])
 
 	/* run the application */
 	g_print ("Evolution-jescs up and running!\n");
+	if (getenv ("EVOLUTION_JESCS_VERBOSE_DEBUG")) {
+                /* Redirect stderr to stdout and make it line-buffered
+                 * rather than block-buffered, for ease of debug
+                 * redirection.
+                 */
+                dup2 (STDOUT_FILENO, STDERR_FILENO);
+                setvbuf (stdout, NULL, _IOLBF, 0);
+                setvbuf (stderr, NULL, _IOLBF, 0);
+                printf ("EVOLUTION_JESCS_VERBOSE_DEBUG=1\n");
+        }
+
 	bonobo_main ();
 
 	e_passwords_shutdown ();
